; -*-Lisp-*-
;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
;
;               Hewlett-Packard Company Confidential
;
; SCCS:         %W%
; Description:  read string/number/pick functions
; Author:       Markus Kuehl
; Created:      Wed Oct 19 08:52:09 1994
; Modified:     Thu Oct 20 11:28:33 1994 (Markus Kuehl) markusk@hpbeo96
; Language:     Lisp
; Package:      ui
;
; (C) Copyright 1994 Hewlett-Packard GmbH, MDD all rights reserved.
;
;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;

(in-package :ui)
(use-package '(:frame2 :uib))

(export '(read-string
	  read-number
	  read-length
	  read-pickpoint
	  read-2dcatchpoint
	  read-3dcatchpoint))

;;===== read-string ===========================================================

(defiaction read_string   ;; DEFine Interrupt ACTION
  (         ;; list of local variables with initial value
   (prompt "Enter a string")
   (result nil)
   )

  ( ;; Description of State Machine
   (get-prompt                           ;; State Name
    nil                                  ;; State Form
    "Enter the prompt for read_string"   ;; State Prompt
    nil                                  ;; Help Update Symbol
    
    ;; Transitions:
    ;; 1. Transition Pattern
    ;; 2. Transition Form
    ;; 3. Personality Update Symbol
    ;; 4. Next State
    (:string (setq prompt (last_pm)) nil get-string) 
    )
   
   (get-string
    nil
    prompt
    nil

    (:string (setq result (last_pm)) nil end)
    )

   (end
    (if (positive_terminate)
        (set_action_result result)
      (set_action_result nil))
    nil
    nil
    )
   )
  ()  ;; local Functions
)

;;-----------------------------------------------------------------------------

(defun read-string (prompt)
  (getres (read_string prompt :interactive-rest)))

;;===== read-number ===========================================================

(defiaction read_number
  ((prompt "Enter a number")
   (result nil))
  ((get-prompt  nil "Enter the prompt for read_number" nil
		(:string (setq prompt (last_pm)) nil get-number)) 
   (get-number nil prompt nil
	       (:number (setq result (last_pm)) nil end))
   (end (if (positive_terminate)
	    (set_action_result result)
	  (set_action_result nil)) nil nil))
  ())

;;-----------------------------------------------------------------------------

(defun read-number (prompt)
  (getres (read_number prompt :interactive-rest)))

;;===== read-length ===========================================================
;; returns entered length (number) converted to mm

(defiaction read_length
  ((prompt "Enter a length")
   (result nil))
  ((get-prompt  nil "Enter the prompt for read_length" nil
		(:string (setq prompt (last_pm)) nil get-length)) 
   (get-length nil prompt nil
	       (:length (setq result (last_state_input)) nil end))
   (end (if (positive_terminate)
	    (set_action_result result)
	  (set_action_result nil)) nil nil))
  ())

;;-----------------------------------------------------------------------------

(defun read-length (prompt)
  (getres (read_length prompt :interactive-rest)))

;;===== read-pickpoint ========================================================

(defiaction read_pickpoint
  ((prompt "Pick a point")
   (result nil))
  ((get-prompt  nil "Enter the prompt for read_pickpoint" nil
		(:string (setq prompt (last_pm)) nil get-pickpoint)) 
   (get-pickpoint nil prompt nil
		  (:wcvp (setq result (last_state_input)) nil end))
   (end (if (positive_terminate)
	    (set_action_result result)
	  (set_action_result nil)) nil nil))
  ())

;;-----------------------------------------------------------------------------

(defun read-pickpoint (prompt)
  (getres (read_pickpoint prompt :interactive-rest)))

;;===== read-2dcatchpoint =====================================================

(defiaction read_2dcatchpoint
  ((prompt "Pick or enter a 2D point")
   (result nil))
  ((get-prompt  nil "Enter the prompt for read_2dcatchpoint" nil
		(:string (setq prompt (last_pm)) nil get-2dcatchpoint)) 
   (get-2dcatchpoint (elan::start-2d-operation) prompt nil
		     (:catch-crd2d (setq result (last_state_input)) nil end))
   (end (progn (if (positive_terminate)
		   (set_action_result result)
		 (set_action_result nil))
	       (elan::end-2d-operation)) nil nil))
  ())

;;-----------------------------------------------------------------------------

(defun read-2dcatchpoint (prompt)
  (getres (read_2dcatchpoint prompt :interactive-rest)))

;;===== read-3dcatchpoint =====================================================

(defiaction read_3dcatchpoint
  ((prompt "Pick or enter a 3D point")
   (result nil))
  ((get-prompt  nil "Enter the prompt for read_3dcatchpoint" nil
		(:string (setq prompt (last_pm)) nil get-3dcatchpoint)) 
   (get-3dcatchpoint (elan::start-3d-operation) prompt nil
		     (:catch-crd3d (setq result (last_state_input)) nil end))
   (end (progn (if (positive_terminate)
		   (set_action_result result)
		 (set_action_result nil))
	       (elan::end-3d-operation)) nil nil))
  ())

;;-----------------------------------------------------------------------------

(defun read-3dcatchpoint (prompt)
  (getres (read_3dcatchpoint prompt :interactive-rest)))

