; -*-Lisp-*-
;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
;
;               Hewlett-Packard Company Confidential
;
; Description:  Rohr-Erzeugung durch 3 Punkte und einen Radius (entlang Kurve)
; Author:       Manfred Rose
; Created:      Die, 24. Jan 1995
; Modified:     
; Language:     Lisp
; Package:      custom
;
; (C) Copyright 1994 Hewlett-Packard GmbH, SF37 all rights reserved.
;
;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;

;; NOTE: 
;;   Dependent files:   read_ar.lsp
;;     => load this file first !

;;- - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -


(in-package :custom)
(use-package '(:frame2 :frame2-ui :ui :uib :ppg-ui))

;; :custom is a new package => use it in SolidDesigners default package
(use-package :custom :mei)

(export '(create-rohr))

;;- - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -

(setq *rohrdurchmesser_halb* 6)

;;- - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -
(defun 3d_2d (pnt &aux 
	(pnt2 (pos-transformation-3d pnt (inverse-transformation-3d (parcel_path_xform (inq_curr_nparcel NWP_TYPE))))))
	(gpnt2d (gpnt3d_x pnt2) (gpnt3d_y pnt2))
)
;;- - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -

;;;; Randbedingungen:
;;   wegen vec-normalize: kein 0,0,0-Vektor erlaubt!
;;   keine bergnge Rohrbogen zu Bogen direkt ohne Geradenstck

(defun create-rohr (radius p1 p2 p3 rohrdurchmesser_halb)
   (let (p1_2d
	 p2_2d
	 p3_2d
	 p1p2_2d
	 p3p2_2d
	 p1_norm
	 p3_norm
  	 normal
 	 len
	 ae_winkel
	 laenge_kurz
	 pt_1
	 wandstaerke
 	 radiusvek_4
 	 radiusvek_5)

;  1. Einstellung der Umgebung !
;;	(set_show "1" :3dgeo :off)
;;	(set_show "1" :3dgeo :on )
;	(set_show "1" :workplanes_border :off )
;	(set_show "1" :workplanes_local_axis :off )
;	(set_show "1" :workplanes_label :off )
;	(set_show "1" :workplanes_2dgeo_vertices :on  )
;	(set_show "1" :workplanes_hidden :off )
	(setq wandstaerke 1)

;  2. Erstellung der Arbeitsebene, in der die Leitkurve mit ihren 3 Punkten liegt!
	(create_workplane :new :name "AE_Leitkurve" :pt_pt_pt p1 p2 p3)

;  3.  Projektion der 3D-Punkte in aktuelle Arbeitsebene
	(setf p1_2d (3d_2d p1))
	(setf p2_2d (3d_2d p2))
	(setf p3_2d (3d_2d p3))
	
	(POLYGON p1_2d p2_2d p3_2d) 
	(put-buffer "complete")

;  4.  Verrundung der Leitkurve!
	(FILLET :CREATE :FILLET_RADIUS radius p2_2d)
	(put-buffer "complete")

;  5. Ermittlung des Winkels zwischen den beiden Vektoren
        (setq p1_norm (vec-subtract p1 p2))
        (setq p3_norm (vec-subtract p3 p2))
        (setq p1_norm (vec-normalize p1_norm))
        (setq p3_norm (vec-normalize p3_norm))
	(setq ae_winkel (vec-angle p1_norm p3_norm))
	(setq ae_winkel (/ (* 180 ae_winkel) PI))
;	(setq ae_winkel (/ (* 180 (vec-angle p1_norm p3_norm)) PI))
	(setq dr_winkel (- 180 ae_winkel))

;  6. Ermittlung des Tangentialpunktes, d.h. Endpunkt des 1. Rohrstcks und
;     Beginn des Rohrbogens!
	(current_wp "/AE_Leitkurve")
	(setq dr_winkel_2 (/ dr_winkel 2))
	;;  (setf sin_in_gr (sin dr_winkel_2))  funktioniert nicht, da Winkel in Rad!!!
	;;  Umrechnung von Rad  (Bogenmass) in Grad beachten:
	(setq win_in_gr (tan (/ (* PI dr_winkel_2) 180 )))
	(setq laenge_kurz (* radius win_in_gr))
;	(display laenge_kurz)  Verkrzung der Lnge p1 p2 zum Tangentialpunkt !!!

;  7. Autom. Erzeugung des Rohr-Profils!
 ; Arbeitsebene dazu :
 	(create_workplane :new :name "AE_Profil_des_Rohrs" :norm_wp :ref_wp "/AE_Leitkurve" :origin p1 :normal :two_pt p1 p2)
 ; Profildefintion des Rohres :
 	(CIRCLE :CEN_RAD 0,0 rohrdurchmesser_halb)
 	(put-buffer "complete")


;  8. Erzeugung des ersten Rohr-Stcks!
 ; Ermittlung der Extrusionlnge:
	(current_wp "/AE_Leitkurve")
     (setq p1p2_2d (sub_gpnt2d p2_2d p1_2d))
     (setq len (sqrt (+ (* (gpnt2d_x p1p2_2d) (gpnt2d_x p1p2_2d))
 		       (* (gpnt2d_y p1p2_2d) (gpnt2d_y p1p2_2d)))))
	(setq pt_1 (- len laenge_kurz))
	(current_wp "/AE_Profil_des_Rohrs")
 	(extrude :part "/R1" :distance pt_1)
 
;  9. Autom. Neuplazierung des Rohr-Profils zum Drehen!
	(position_wp "/AE_Profil_des_Rohrs" :par_wp :offset pt_1 )

; 10. Erzeugung des zweiten Rohr-Stcks, d.h. des Rohrbogens!
	(current_wp "/AE_Profil_des_Rohrs")
	(turn :part "/R2" :axis :vert radius :rotation_angle dr_winkel)

	(delete_3d  "/AE_Profil_des_Rohrs")

; 11. Autom. Erzeugung des Rohr-Profils-3 !
 ; Arbeitsebene dazu :
 	(create_workplane :new :name "AE_Profil_des_Rohrs_2" :norm_wp :ref_wp "/AE_Leitkurve" :origin p3 :normal :two_pt p3 p2)
 ; Profildefintion des Rohres :
 	(CIRCLE :CEN_RAD 0,0 rohrdurchmesser_halb)
 	(put-buffer "complete")
 
; 12. Erzeugung des zweiten linearen, des dritten Rohr-Stcks!
 ; Ermittlung der Extrusionlnge:
	(current_wp "/AE_Leitkurve")
     (setq p3p2_2d (sub_gpnt2d p2_2d p3_2d))
     (setq len (sqrt (+ (* (gpnt2d_x p3p2_2d) (gpnt2d_x p3p2_2d))
 		       (* (gpnt2d_y p3p2_2d) (gpnt2d_y p3p2_2d)))))
	(setq pt_1 (- len laenge_kurz))
	(current_wp "/AE_Profil_des_Rohrs_2")
 	(extrude :part "/R3" :distance pt_1)
 	(put-buffer "complete")
 
;	(delete_3d "/AE_Profil_des_Rohrs" "/AE_Profil_des_Rohrs_2")
	(delete_3d "/AE_Profil_des_Rohrs_2")

;	(unite_3d :blank "/R1" :tools :start "/R2" "/R3" :select_done :keep_tools :no)
;;	(shell_3d :select_part "/R1" :offset wandstaerke :open_face :start p1 p3 :select_done)

))



;;-----------------------------------------------------------------------------
;;-----------------------------------------------------------------------------
;;-----------------------------------------------------------------------------
;;-----------------------------------------------------------------------------



(in-package :custom)
(use-package '(:frame2 :frame2-ui :ui :uib))

;; :custom is a new package => use it in SolidDesigners default package
(use-package :custom :mei)

(export '(rohr-radius-toggle-pushed
	  rohr-point1-toggle-pushed
	  rohr-point2-toggle-pushed
	  rohr-point3-toggle-pushed))

;;----- global variables ------------------------------------------------------

(defvar *rohr-radius*)
(defvar *rohr-p1*)
(defvar *rohr-p2*)
(defvar *rohr-p3*)
(defvar *rohrdurchmesser*)
(defvar *rohrdurchmesser_halb*)

;;----- Create the User Interface ---------------------------------------------

(create-right-menu-dialog "ROHR"         ;; Name of Dialog
			  "Erstelle Rohr"  ;; Toplabel
   '(
     (:heterogeneous  "ROHR-OPTIONS"
      (
       (:toggle-data "RADIUS"
		     "Radius"
		     :length
		     :half
		     "(rohr-radius-toggle-pushed)"  ;; this has to go through
		     ;; the Input Buffer, because the function requires
		     ;; data interactively !
		     :cancel-and-restart-command
		     (rohr-radius-return)  ;; direct LISP function call
		     t
       )
       (:toggle-data "PUNKT1"
		     "Punkt 1"
		     :info
		     :half
		     "(rohr-point1-toggle-pushed)"
		     :cancel-and-restart-command
		     nil  ;; not editable => no return action necessary
		     t
       )
       (:toggle-data "PUNKT2"
		     "Punkt 2"
		     :info
		     :half
		     "(rohr-point2-toggle-pushed)"
		     :cancel-and-restart-command
		     nil  ;; not editable => no return action necessary
		     t
       )
       (:toggle-data "PUNKT3"
		     "Punkt 3"
		     :info
		     :half
		     "(rohr-point3-toggle-pushed)"
		     :cancel-and-restart-command
		     nil  ;; not editable => no return action necessary
		     t
       )
      )
     )
     (:bottom)
    )
   :close-action '(hide-rohr-dialog)
)

;;- - - adapt Ok, Cancel, Help callbacks  - - - - - - - - - - - - - - - - - - -

(change_wid_property_f "ROHR-BL-COMPLETE-PB"
		       :sensitive     :true
		       :push          '(rohr-ok-pushed))

(change_wid_property_f "ROHR-BL-CANCEL-PB"
		       :sensitive     :true
		       :push          '(rohr-cancel-pushed))

(change_wid_property_f "ROHR-BL-HELP-PB"
		       :push          '(rohr-help-pushed))

(change_wid_property_f "ROHR-FO"
		       :borderColor   ui::*HighlightColor*)

;;----- Add a button to the toolbox -------------------------------------------

(when (widget-id "TOOLBOX-COMMANDS-ERSTELLE_ROHR-TB")
  (remove_toolbox_button :label "Erstelle Rohr"))

(add_toolbox_button :label "Erstelle Rohr"
		    :action '(progn
			       (show-and-initialize-rohr-dialog)
			       (reset-toggle "TOOLBOX-COMMANDS-ERSTELLE_ROHR-TB")))


;;----- Widget Callback functions ---------------------------------------------

(defun show-and-initialize-rohr-dialog ()
  ;; initialize radius datafield
  (reset-toggle "ROHR-OPTIONS-OPT-CONT-RADIUS-TB")
  (setq *rohr-radius* nil)
  (setq *rohr-radius* 25)
  (update-toggle-data "ROHR-OPTIONS" "RADIUS" nil 
		      :direct-value *rohr-radius*)
  ;; initialize Point 1 data field
  (reset-toggle "ROHR-OPTIONS-OPT-CONT-PUNKT1-TB")
  (setq *rohr-p1* nil)
;  (setq *rohr-p1* (gpnt3d 0 0 0))
  (update-toggle-data "ROHR-OPTIONS" "PUNKT1" nil 
		      :direct-value *rohr-p1*)
  ;; initialize Point 2 data field
  (reset-toggle "ROHR-OPTIONS-OPT-CONT-PUNKT2-TB")
  (setq *rohr-p2* nil)
;  (setq *rohr-p2* (gpnt3d 10 10 10))
  (update-toggle-data "ROHR-OPTIONS" "PUNKT2" nil 
		      :direct-value *rohr-p2*)
  ;; initialize Point 3 data field
  (reset-toggle "ROHR-OPTIONS-OPT-CONT-PUNKT3-TB")
  (setq *rohr-p3* nil)
;  (setq *rohr-p3* (gpnt3d 15 30 50))
  (update-toggle-data "ROHR-OPTIONS" "PUNKT3" nil 
		      :direct-value *rohr-p3*)
  ;; show SLOT Dialog
  (show-shell-menu-box "ROHR" :shell-position '("TOOLBOX-DS"  ;; Reference Wid
						:bottomleft   ;; Attachment
						0             ;; x offset
						0))           ;; y offset
  )

(defun hide-rohr-dialog ()
  (hide-dialog "ROHR")
  (put-buffer "cancel_all"))

;;- - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -

(defun rohr-radius-toggle-pushed ()
  (set-toggle "ROHR-OPTIONS-OPT-CONT-RADIUS-TB")
  (setq *rohr-radius* (read-length "Geben Sie den Radius der Leitkurve fr den Rohrbogen an."))
  (reset-toggle "ROHR-OPTIONS-OPT-CONT-RADIUS-TB")
  (update-toggle-data "ROHR-OPTIONS" "RADIUS" nil 
		      :direct-value *rohr-radius*)
  (values))

(defun rohr-radius-return ()
  (if (is-toggle-set "ROHR-OPTIONS-OPT-CONT-RADIUS-TB")
      ;; THEN: get text widget value and put it into the Input Buffer
      (put-buffer (get_text_wid_value_f "ROHR-OPTIONS-OPT-CONT-RADIUS-TX"))
    ;; ELSE
    (progn
      (setq *rohr-radius* (read-from-string (get_text_wid_value_f
					    "ROHR-OPTIONS-OPT-CONT-RADIUS-TX")))
      (update-toggle-data "ROHR-OPTIONS" "RADIUS" nil 
			  :direct-value *rohr-radius*)))
  (values))

;;- - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -

(defun rohr-point1-toggle-pushed ()
  (set-toggle "ROHR-OPTIONS-OPT-CONT-PUNKT1-TB")
  (setq *rohr-p1* (read-3dcatchpoint "Geben Sie den ersten Leitkurven-Punkt fr das Rohr ein."))
  (reset-toggle "ROHR-OPTIONS-OPT-CONT-PUNKT1-TB")
  (when *rohr-p1*
    ;; show point1 with 2 digits after the decimal point: e.g. 6.43,123.29,154.55
    (update-toggle-data "ROHR-OPTIONS" "PUNKT1" nil 
			:direct-value (fix_mesg "{1},{2},{3}"
						(num-to-str (gpnt3d_x *rohr-p1*) 2)
						(num-to-str (gpnt3d_y *rohr-p1*) 2)
						(num-to-str (gpnt3d_z *rohr-p1*) 2))))
  (values))

;;- - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -

(defun rohr-point2-toggle-pushed ()
  (set-toggle "ROHR-OPTIONS-OPT-CONT-PUNKT2-TB")
; (setq *rohr-p2* (read-2dcatchpoint "Pick or enter the second Center Point."))
  (setq *rohr-p2* (read-3dcatchpoint "Geben Sie den zweiten Leitkurven-Punkt fr das Rohr ein."))
  (reset-toggle "ROHR-OPTIONS-OPT-CONT-PUNKT2-TB")
  (when *rohr-p2*
    (update-toggle-data "ROHR-OPTIONS" "PUNKT2" nil 
			:direct-value (fix_mesg "{1},{2},{3}"
						(num-to-str (gpnt3d_x *rohr-p2*) 2)
						(num-to-str (gpnt3d_y *rohr-p2*) 2)
						(num-to-str (gpnt3d_z *rohr-p2*) 2))))
  (values))

;;- - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -

(defun rohr-point3-toggle-pushed ()
  (set-toggle "ROHR-OPTIONS-OPT-CONT-PUNKT3-TB")
  (setq *rohr-p3* (read-3dcatchpoint "Geben Sie den dritten Leitkurven-Punkt fr das Rohr ein."))
  (reset-toggle "ROHR-OPTIONS-OPT-CONT-PUNKT3-TB")
  (when *rohr-p3*
    (update-toggle-data "ROHR-OPTIONS" "PUNKT3" nil 
			:direct-value (fix_mesg "{1},{2},{3}"
						(num-to-str (gpnt3d_x *rohr-p3*) 2)
						(num-to-str (gpnt3d_y *rohr-p3*) 2)
						(num-to-str (gpnt3d_z *rohr-p3*) 2))))
  (values))

;;- - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -
	     
(defun rohr-cancel-pushed ()
  (hide-rohr-dialog))
	
(defun rohr-help-pushed ()
  (put-buffer "info :keyword \"line\""))  ;; only a guess ;-)

;;=============================================================================

(defun rohr-ok-pushed ()
  (if (and *rohr-radius* *rohr-p1* *rohr-p2* *rohr-p3* *rohrdurchmesser_halb*)
      ;; THEN
      (create-rohr *rohr-radius* *rohr-p1* *rohr-p2* *rohr-p3* *rohrdurchmesser_halb*)
    ;; ELSE
    (display_error "Rohr-Leitlinie nicht erstellt, da noch einige Daten fehlen!"))
  (hide-dialog "ROHR")
  (update-screen))

